package zs.model;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class Shipment {
	private String tracking;
	private Recipient recipient;
	private float weightInKg;
	private List<Delivery> deliveries = new ArrayList<Delivery>();

	public Shipment(String tracking, Recipient recipient, float weightInKg) {
		if (tracking == null || tracking.trim().isEmpty())
			throw new IllegalArgumentException("tracking");
		if (recipient == null)
			throw new IllegalArgumentException("recipient");
		if (weightInKg <= 0)
			throw new IllegalArgumentException("weightInKg");
		
		this.tracking = tracking;
		this.recipient = recipient;
		this.weightInKg = weightInKg;
	}

	public String getTracking() {
		return tracking;
	}

	public Recipient getRecipient() {
		return recipient;
	}
	
	public float getWeightInKg() {
		return weightInKg;
	}
	
	public List<Delivery> getDeliveries() {
		return Collections.unmodifiableList(deliveries);
	}
	
	public boolean isDelivered() {
		return deliveries.size() > 0 && 
			   deliveries.get(deliveries.size() - 1) instanceof SucceededDelivery;
	}

	public SucceededDelivery createSucceededDelivery(LocalDateTime dateTime, String notes, String recipientSign) {
		if (dateTime == null)
			throw new IllegalArgumentException("dateTime");
		if (recipientSign == null || recipientSign.trim().isEmpty())
			throw new IllegalArgumentException("recipientSign");
		
		checkNotDelivered();
		
		SucceededDelivery delivery = new SucceededDelivery(dateTime, notes, recipientSign);
		deliveries.add(delivery);
		return delivery;
	}
	
	public FailedDelivery createFailedDelivery(LocalDateTime dateTime, String notes) {
		if (dateTime == null)
			throw new IllegalArgumentException("dateTime");
		
		checkNotDelivered();
		
		FailedDelivery delivery = new FailedDelivery(dateTime, notes);
		deliveries.add(delivery);
		return delivery;
	}
	
	private void checkNotDelivered() {
		if (isDelivered())
			throw new InvalidOperationException("Already delivered");
	}
	
	@Override
	public String toString() {
		return getRecipient().toString();
	}
}
